/*
 *    Blimp.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
*/

#include "defs.h"
#include "Blimp.h"
#include "BlimpSystem.h"
#include <cstdio>
#include <iostream>
#include <unistd.h>
using namespace std;

extern BlimpSystem blimpSystem;

Blimp::Blimp()
{
    controller = new FlightController;
    id = TEAM_ID;
    usec = MAIN_THREAD_SLEEP;
}

Blimp::~Blimp()
{
}

PositionSensor *Blimp::getPositionSensor()
{
    return &posSensor;
}

FlightPlan *Blimp::getFlightPlan()
{
    return &plan;
} 

void Blimp::setCharacteristics(char *filename)
{
    controller->setCharacteristics(filename);
}

void Blimp::startAutomaticFlight()
{
    controller->startAutomaticFlight();
    Goal goal = plan.getFirstGoal();
    controller->setGoal(goal);

    bool isFinalGoal = false;
    while (!isFinalGoal) {
        bool hasArrived = false;
        bool passed = false;

        while (!hasArrived && !passed) {
            flight();
            showStatus();
            hasArrived = controller->hasArrived();
            if (!hasArrived) {
                passed = controller->passedBy();
            }
        }

        if (hasArrived || passed) {
            isFinalGoal = plan.isFinalGoal();

	    if (!isFinalGoal) {
                goal = plan.getNextGoal();
                controller->setGoal(goal);
	    }
	}
    }
    stop();
}

void Blimp::testMotorSensors(MotorPower *power)
{
    motor.rotate(id, power);
    getSensorInfo();
    showStatus();
}

void Blimp::flight()
{
    getSensorInfo();
    MotorPower *power = controller->getMotorPower();
    motor.rotate(id, power);
    usleep(usec);
}

void Blimp::showStatus()
{
    showAltitudeInfo();
    showPositionInfo();
    showDirectionInfo();

    bool playMode = blimpSystem.isAutomaticFlight();
    if (playMode) {
        controller->showStatus();
    }
}

void Blimp::stop()
{
    motor.stop(id);
}

void Blimp::getSensorInfo()
{
    int alt;
    int dir;
    Position pos;

    alt = altSensor.getData();
    dir = dirSensor.getData();
    pos = posSensor.getData();
    controller->setSensorInfo(alt, dir, pos);
}

void Blimp::showAltitudeInfo()
{
    int alt = controller->getAltitudeInfo();
    cout << "ALT DATA: " << alt << "\n";
}

void Blimp::showPositionInfo()
{
    Position pos = controller->getPositionInfo();
    cout << "POS DATA: (" << pos.x << "," << pos.y << "," << pos.z << ")\n";
}

void Blimp::showDirectionInfo()
{
    int dir = controller->getDirectionInfo();
    cout << "DIR DATA: " << dir << "\n";
}

void Blimp::setFlightController(FlightController *cnt)
{
    controller = cnt;
}
