/*
 *    BlimpSystem.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "BlimpSystem.h"
#include <cstdio>
#include <iostream>
#include <unistd.h>
using namespace std;

void BlimpSystem::setStatus(int _status)
{
    status = _status;
}

void BlimpSystem::setUSSensors(char *filename)
{
    FILE *fp = fopen(filename, "r");
    if (fp == NULL) {
        cout << "File not found: " << filename << "\n";
	return;
    }

#ifdef DEBUG
    cout << "READING CHARACT: " << filename << "\n";
#endif
    PositionSensor *posSensor = blimp->getPositionSensor();
    USSensor us = readUSSensorInfo(fp);
    while (us.id != INVALID_VALUE) {
        posSensor->setUSSensor(us);
        us = readUSSensorInfo(fp);
    }
    fclose(fp);

    if (status == READY_CH) {
        setStatus(READY_CHUS);
    } else if (status == INITIAL) {
        setStatus(READY_US);
    }
}

USSensor BlimpSystem::readUSSensorInfo(FILE *fp)
{
    USSensor us;
    Position pos;
    int id, x, y, z;
    char buf[256];

    if (fgets(buf, 255, fp) != NULL) {
        sscanf(buf, "%d %d %d %d", &id, &x, &y, &z);
        us.pos.x = x;
        us.pos.y = y;
        us.pos.z = z;
        us.id = id;
    } else {
        us.id = INVALID_VALUE;
    }
    return us;
}

void BlimpSystem::setCharacteristics(char *filename)
{
    blimp->setCharacteristics(filename);

    if (status == READY_US) {
        setStatus(READY_CHUS);
    } else if (status == INITIAL) {
        setStatus(READY_CH);
    }
}

void BlimpSystem::setFlightPath(char *filename)
{
    if (status == READY_CHUS) {
        readFlightPath(filename);
        setStatus(READY_ALL);
    } else {
        cout << "Please input characteristics and us sensor information!\n";
    }
}

void BlimpSystem::readFlightPath(char *filename)
{
    FILE *fp = fopen(filename, "r");
    if (fp == NULL) {
        cout << "File not found: " << filename << "\n";
	return;
    }

#ifdef DEBUG
    cout << "READING FLIGHT PATH: " << filename << "\n";
#endif
    int goaln = 0;
    FlightPlan *plan = blimp->getFlightPlan();
    Goal goal = readGoalInfo(fp);
    while (goal.period != INVALID_VALUE) {
        goaln++;
        cout << "  GOAL: " << goaln << " ("
             << goal.pos.x << "," << goal.pos.y << "," << goal.pos.z << ") "
             << goal.period << "\n";
        plan->addGoal(goal);
        goal = readGoalInfo(fp);
    }
    fclose(fp);
}

Goal BlimpSystem::readGoalInfo(FILE *fp)
{
    Goal goal;
    int x, y, z, t;
    char buf[256];

    if (fgets(buf, 255, fp ) != NULL) {
        sscanf(buf, "%d %d %d %d", &x, &y, &z, &t);
        goal.pos.x = x;
        goal.pos.y = y;
        goal.pos.z = z;
	goal.period = t;
    } else {
        goal.period = INVALID_VALUE;
    }
    return goal;
}

void BlimpSystem::startAutomaticFlight()
{
    if (status == READY_ALL) {
        setStatus(PLAYING);
        blimp->startAutomaticFlight();
        displayLandingMessage();

        setStatus(READY_ALL);
    } else {
        printf("Lack of information for automatic flight!\n");
    }
}

void BlimpSystem::displayLandingMessage()
{
    GUIshowLandingMessage("<h1>Goooooooooooooal!!</h1>");
    cout << "**** Just landing! ****\n";
}

void BlimpSystem::startTestFlight()
{
    if (status == READY_US || status == READY_CHUS || status == READY_ALL) {
        int history = status;
        setStatus(TESTING);
        MotorPower *power = showMotorPowerPrompt();
        while (power != NULL) {
            blimp->testMotorSensors(power);
            power = showMotorPowerPrompt();
        }
        setStatus(history);
        cout << "Test mode terminated.\n";
    } else {
        printf("Please input us sensor information!\n");
    }
}

MotorPower *BlimpSystem::showMotorPowerPrompt()
{
    int left, right, upper;
    MotorPower *power = new MotorPower;

    cout << "Input motor power: ";
    cin >> power->upper;
    if (power->upper > 99) {
        return NULL;
    }
    cin >> power->left;
    cin >> power->right;

    return power;
}

bool BlimpSystem::isAutomaticFlight()
{
    return status == PLAYING;
}

void BlimpSystem::setBlimp(Blimp *_blimp)
{
    blimp = _blimp;
}

void BlimpSystem::setFlightController(FlightController *cnt)
{
    blimp->setFlightController(cnt);
}

void BlimpSystem::setSleep(unsigned long usec)
{
    blimp->usec = usec;
}
