/*
 *    BlimpSystem.h
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#ifndef _FSEL_BLIMPSYSTEM_DEFINED
#define _FSEL_BLIMPSYSTEM_DEFINED

#include <cstdio>
#include "Position.h"
#include "Goal.h"
#include "MotorPower.h"
#include "Blimp.h"
#include "FlightPlan.h"
#include "FlightController.h"

#define INITIAL 0
#define READY_CH 1
#define READY_US 2
#define READY_CHUS 3
#define READY_ALL 4
#define PLAYING 5
#define TESTING 6

class BlimpSystem {
private:
    Blimp *blimp;
    int status;
    USSensor readUSSensorInfo(FILE *fp);
    void readFlightPath(char *filename);
    Goal readGoalInfo(FILE *fp);
protected:
    MotorPower *showMotorPowerPrompt();
public:
    void setStatus(int _status);
    void setUSSensors(char *filename);
    void setCharacteristics(char *filename);
    void setFlightPath(char *filename);
    void startAutomaticFlight();
    void displayLandingMessage();
    void startTestFlight();
    bool isAutomaticFlight();
    void setBlimp(Blimp *_blimp);
    void setFlightController(FlightController *cnt);
    void setSleep(unsigned long usec);
};

#endif
