/*
 *    BlindFlight.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "BlindFlight.h"
#include <cstdio>
#include <iostream>
using namespace std;

void BlindFlight::setCharacteristics(char *filename)
{
    char name[256];
    strcpy(name, "blind_");
    strcat(name, filename);

    FILE *fp = fopen(name, "r");
    if (fp == NULL) {
        cout << "File not found: " << name << "\n";
        return;
    }

#ifdef DEBUG
    cout << "READING CHARACT: " << name << "\n";
#endif

    charact->distMap = readCharacteristics(fp);
    charact->upperMap = readCharacteristics(fp);
    charact->rightMap = readCharacteristics(fp);
    charact->leftMap = readCharacteristics(fp);

    fclose(fp);
}

static int flag = 0; /* β */

bool BlindFlight::hasArrived(Position *pos, Position *gpos)
{
    bool diff;
    cout << "BlindMode: ";
    int dz = abs(gpos->z - pos->z);
    cout << "GOAL = (" << gpos->x << " " << gpos->y << " "<< gpos->z << "): " << dz << "\n";
    diff = dz < ALT_ERROR_IN_ARRIVAL;
    if (diff || (pos->z - gpos->z) > 0) {
	if (flag == 0) {
	    flag = 1;
	} else {
	    diff = false;
	}
    }
    return diff;
}

MotorPower *BlindFlight::getMotorPower(Position *pos, int dir, Position *gpos)
{
    int altdiff = getAltitude(pos->z, gpos->z);
    return calculate(altdiff);
}

MotorPower *BlindFlight::calculate(int altdiff)
{
    static MotorPower power;

    altdiff /= 10;
    if (charact->upperMap.count(abs(altdiff)) == 1) {
        power.upper = charact->upperMap[abs(altdiff)];
    } else {
        power.upper = MAX_MOTOR_POWER;
    }
    if (altdiff < 0) {
	power.upper *= -1;
    }
    power.upper += HOVER_MOTOR_POWER;

    if (altdiff <= ALT_ERROR_IN_ARRIVAL) {
        power.left = MIN_MOTOR_POWER * BLINDMODE_HORIZONTAL_MOVE_WEIGHT;
    } else {
        power.left = 0;
    }
    power.right = power.left;

    return &power;    
}
