/*
 *    FSELForm.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "FSELForm.h"
#include "defs.h"

#include <qapplication.h>
#include <qcolor.h>
#include <qvariant.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qlcdnumber.h>
#include <qtextbrowser.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

/*
 *  Constructs a FSELForm as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
FSELForm::FSELForm( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "FSELForm" );

    QColor color;
    color.setRgb(0, 0, 0);

    groupBox1 = new QGroupBox( this, "groupBox1" );
    groupBox1->setGeometry( QRect( 30, 40, 380, 160 ) );

    textLabel2 = new QLabel( groupBox1, "textLabel2" );
    textLabel2->setGeometry( QRect( 20, 30, 16, 20 ) );

    textLabel2_3 = new QLabel( groupBox1, "textLabel2_3" );
    textLabel2_3->setGeometry( QRect( 260, 30, 16, 20 ) );

    lCDX = new QLCDNumber( groupBox1, "lCDX" );
    lCDX->setGeometry( QRect( 20, 50, 100, 40 ) );
    lCDX->setSegmentStyle(QLCDNumber::Filled);
    lCDX->setPaletteForegroundColor(color);

    lCDY = new QLCDNumber( groupBox1, "lCDY" );
    lCDY->setGeometry( QRect( 140, 50, 100, 40 ) );
    lCDY->setSegmentStyle(QLCDNumber::Filled);
    lCDY->setPaletteForegroundColor(color);

    lCDZ = new QLCDNumber( groupBox1, "lCDZ" );
    lCDZ->setGeometry( QRect( 260, 50, 100, 40 ) );
    lCDZ->setSegmentStyle(QLCDNumber::Filled);
    lCDZ->setPaletteForegroundColor(color);

    lCDY_2 = new QLCDNumber( groupBox1, "lCDY_2" );
    lCDY_2->setGeometry( QRect( 140, 110, 100, 40 ) );
    lCDY_2->setSegmentStyle(QLCDNumber::Filled);
    lCDY_2->setPaletteForegroundColor(color);

    textLabel2_2 = new QLabel( groupBox1, "textLabel2_2" );
    textLabel2_2->setGeometry( QRect( 140, 30, 16, 20 ) );

    textLabel2_2_3 = new QLabel( groupBox1, "textLabel2_2_3" );
    textLabel2_2_3->setGeometry( QRect( 90, 120, 48, 20 ) );

    textLabel1 = new QLabel( this, "textLabel1" );
    textLabel1->setGeometry( QRect( 190, 10, 67, 20 ) );
    textLabel1->setFrameShape( QLabel::NoFrame );
    textLabel1->setFrameShadow( QLabel::Plain );

    groupBox2 = new QGroupBox( this, "groupBox2" );
    groupBox2->setGeometry( QRect( 30, 210, 380, 100 ) );

    textLabel2_4 = new QLabel( groupBox2, "textLabel2_4" );
    textLabel2_4->setGeometry( QRect( 20, 30, 27, 20 ) );

    textLabel2_2_2 = new QLabel( groupBox2, "textLabel2_2_2" );
    textLabel2_2_2->setGeometry( QRect( 140, 30, 40, 20 ) );

    textLabel2_3_2 = new QLabel( groupBox2, "textLabel2_3_2" );
    textLabel2_3_2->setGeometry( QRect( 260, 30, 49, 20 ) );

    lCDLEFT = new QLCDNumber( groupBox2, "lCDLEFT" );
    lCDLEFT->setGeometry( QRect( 140, 50, 100, 40 ) );
    lCDLEFT->setSegmentStyle(QLCDNumber::Filled);
    lCDLEFT->setPaletteForegroundColor(color);

    lCDRIGHT = new QLCDNumber( groupBox2, "lCDRIGHT" );
    lCDRIGHT->setGeometry( QRect( 260, 50, 100, 40 ) );
    lCDRIGHT->setSegmentStyle(QLCDNumber::Filled);
    lCDRIGHT->setPaletteForegroundColor(color);

    lCDUP = new QLCDNumber( groupBox2, "lCDUP" );
    lCDUP->setGeometry( QRect( 20, 50, 100, 40 ) );
    lCDUP->setSegmentStyle(QLCDNumber::Filled);
    lCDUP->setPaletteForegroundColor(color);

    Goal = new QTextEdit( this, "Goal" );
    Goal->setGeometry( QRect( 30, 400, 380, 64 ) );
    Goal->setReadOnly(TRUE);

    Status = new QTextEdit( this, "Status" );
    Status->setGeometry( QRect( 100, 320, 300, 72 ) );
    Status->setReadOnly(TRUE);

    textLabel1_2 = new QLabel( this, "textLabel1_2" );
    textLabel1_2->setGeometry( QRect( 30, 340, 55, 20 ) );
    languageChange();
    resize( QSize(440, 480).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    // signals and slots connections
}

/*
 *  Destroys the object and frees any allocated resources
 */
FSELForm::~FSELForm()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void FSELForm::languageChange()
{
    setCaption( tr( "Form1" ) );
    groupBox1->setTitle( tr( "Position" ) );
    textLabel2->setText( tr( "<b>X</b>" ) );
    textLabel2_3->setText( tr( "<b>Z</b>" ) );
    textLabel2_2->setText( tr( "<b>Y</b>" ) );
    textLabel2_2_3->setText( tr( "<b>Angle</b>" ) );
    textLabel1->setText( tr( "<h1><font face=\"Times\">FSEL</font></h1>" ) );
    groupBox2->setTitle( tr( "MotorPower" ) );
    textLabel2_4->setText( tr( "<b>UP</b>" ) );
    textLabel2_2_2->setText( tr( "<b>LEFT</b>" ) );
    textLabel2_3_2->setText( tr( "<b>RIGHT</b>" ) );
    textLabel1_2->setText( tr( "<b>Status</b>" ) );
}

void FSELForm::showAltitudeInfo(int alt) {
    if (alt == INVALID_VALUE) {
	lCDZ->display("---");
    } else {
	lCDZ->display(alt);
    }
}

void FSELForm::showPositionInfo(Position pos) {
    if (pos.x == INVALID_VALUE) {
	lCDX->display("---");
    } else {
	lCDX->display(pos.x);
    }
    if (pos.y == INVALID_VALUE) {
	lCDY->display("---");
    } else {
	lCDY->display(pos.y);
    }
}

void FSELForm::showDirectionInfo(int angle) {
    lCDY_2->display(angle);
}

void FSELForm::showMotorPower(int up, int left, int right) {
    lCDUP->display(up);
    lCDLEFT->display(left);
    lCDRIGHT->display(right);
}

void FSELForm::showStatus(char *text) {
    Status->setText(text);
}

void FSELForm::showLandingMessage(char *str) {
    Goal->setText(str);
}

static FSELForm *w;
static QApplication *a;

void GUIInit(int argc, char **argv) {
    a = new QApplication(argc, argv);
    w = new FSELForm();
    w->show();
//    w->connect(w, SIGNAL(updateValue()), w, SLOT(setValue()));
//    w->connect(w, SIGNAL(updateText()), w, SLOT(setText()));
    a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
}

void *GUIThread(void *param) {
    a->exec();

    return NULL;
}

void GUIshowAltitudeInfo(int alt) {
    w->showAltitudeInfo(alt);
}

void GUIshowPositionInfo(Position pos) {
    w->showPositionInfo(pos);
}

void GUIshowDirectionInfo(int angle) {
    w->showDirectionInfo(angle);
}

void GUIshowMotorPower(int up, int left, int right) {
    w->showMotorPower(up, left, right);
}

void GUIshowStatus(char *str) {
    w->showStatus(str);
}

void GUIshowLandingMessage(char *str) {
    sleep(2); // $B40A4$KDd;_$9$k$N$rBT$D(B
    w->showLandingMessage(str);
}
