/*
 *    FlightController.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
*/

#include "defs.h"
#include "FlightController.h"
#include "FSELForm.h"
#include <iostream>
using namespace std;

FlightController::FlightController()
{
    alt = INVALID_VALUE;
    dir = INVALID_VALUE;
    pos.x = INVALID_VALUE;
    pos.y = INVALID_VALUE;
    pos.z = INVALID_VALUE;

    mode = NULL;  // Waiting for flight
    failures = 0;

    normalFlight = new NormalFlight;
    blindFlight = new BlindFlight;
    lossOfControl = new LossOfControl;
}

void FlightController::setCharacteristics(char *filename)
{
    normalFlight->setCharacteristics(filename);
    blindFlight->setCharacteristics(filename);
    lossOfControl->setCharacteristics(filename);
}

void FlightController::startAutomaticFlight()
{
    mode = blindFlight;
}

void FlightController::setGoal(Goal _goal)
{
    goal = _goal;
    timer.start(_goal.period);
}

bool FlightController::passedBy()
{
    return timer.passedBy();
}

bool FlightController::hasArrived()
{
    return mode->hasArrived(&pos, &goal.pos);
}

void FlightController::setSensorInfo(int _alt, int _dir, Position _pos)
{

    if (_alt < INVALID_VALUE) {
        alt = _alt;
    }
    if (_dir < INVALID_VALUE) {
        dir = _dir;
    }
    if (_pos.x < INVALID_VALUE && _pos.y < INVALID_VALUE && _pos.z < INVALID_VALUE) {
        prevPos = pos;
        pos = _pos;
    }

    adjustPositionInfo(alt);
    changeFlightMode(_pos);
}

void FlightController::changeFlightMode(Position _pos)
{
    bool valid = _pos.x < INVALID_VALUE && _pos.y < INVALID_VALUE && _pos.z < INVALID_VALUE;

    if (mode == blindFlight) {
        if (valid) {
            mode = normalFlight;
        }

    } else if (mode == normalFlight) {
        if (!valid) {
            failures++;
        }
        if (failures > MAX_FAILURES) {
            mode = lossOfControl;
        }
    
    } else if (mode == lossOfControl) {
        if (valid) {
            mode = normalFlight;
            failures = 0;
        }
    }
}

void FlightController::adjustPositionInfo(int _alt)
{
    pos.z = _alt;
}

MotorPower *FlightController::getMotorPower()
{
    prevPower = power;
    MotorPower *p = mode->getMotorPower(&pos, dir, &goal.pos);

    power.left = p->left;
    power.right = p->right;
    power.upper = p->upper;

    return p;
}

int FlightController::getAltitudeInfo()
{
    return alt;
}

int FlightController::getDirectionInfo()
{
    return dir;
}

Position FlightController::getPositionInfo()
{
    return pos;
}

void FlightController::showStatus()
{
    char buf[256];

    buf[0] = '\0';
    if (power.upper > 0) {
        strcat(buf, "GOING UP<br>");
    } else if (power.upper < 0) {
        strcat(buf, "GOING DOWN<br>");
    }

    if (power.left == power.right) {
        strcat(buf, "GOING STRAIGHT<br>");
    } else if (power.left > power.right) {
	strcat(buf, "ROTATE CW<br>");
    } else if (power.right > power.left) {
        strcat(buf, "ROTATE CCW<br>");
    }

    int v0 = power.upper * power.upper +
	power.left * power.left +
	power.right * power.right;
    int v1 = prevPower.upper * prevPower.upper +
	prevPower.left * prevPower.left +
	prevPower.right * prevPower.right;

    if (v0 > v1) {
        strcat(buf, "BOOST<br>");
    } else if (v0 < v1) {
        strcat(buf, "DEBOOST<br>");
    }
    GUIshowStatus(buf);
#ifdef STATUS
    if (power.upper > 0) {
        cout << "GOING UP\n";
    } else if (power.upper < 0) {
        cout << "GOING DOWN\n";
    }

    if (power.left == power.right) {
        cout << "GOING STRAIGHT\n";
    } else if (power.left > power.right) {
        cout << "ROTATE CW\n";
    } else if (power.right > power.left) {
        cout << "ROTATE CCW\n";
    }

    int v0 = power.upper * power.upper +
             power.left * power.left +
             power.right * power.right;
    int v1 = prevPower.upper * prevPower.upper +
             prevPower.left * prevPower.left +
             prevPower.right * prevPower.right;

    if (v0 > v1) {
        cout << "BOOST\n";
    } else if (v0 < v1) {
        cout << "DEBOOST\n";
    }
#endif
}
