/*
 *    FlightController.h
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#ifndef _FSEL_FLIGHTCONTROLLER_DEFINED
#define _FSEL_FLIGHTCONTROLLER_DEFINED

#include "Position.h"
#include "Goal.h"
#include "MotorPower.h"
#include "Characteristics.h"
#include "FlightMode.h"
#include "NormalFlight.h"
#include "BlindFlight.h"
#include "LossOfControl.h"
#include "Motor.h"
#include "Timer.h"

class FlightController {
private:
    Timer timer;
    FlightMode *mode;
    NormalFlight *normalFlight;
    BlindFlight *blindFlight;
    LossOfControl *lossOfControl;
    void adjustPositionInfo(int alt);
protected:
    Goal goal;
    int alt;
    int dir;
    Position pos;
    Position prevPos;
    MotorPower power;
    MotorPower prevPower;
    int failures;
    void changeFlightMode(Position pos);
public:
    FlightController();
    void setCharacteristics(char *filename);
    void FlightController::startAutomaticFlight();
    void setGoal(Goal goal);
    bool passedBy();
    bool hasArrived();
    void setSensorInfo(int alt, int dir, Position pos);
    MotorPower *getMotorPower();
    int getAltitudeInfo();
    int getDirectionInfo();
    Position getPositionInfo();
    void showStatus();
};

#endif
