/*
 *    FlightMode.h
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "FlightMode.h"
#include <map>
#include <cstdio>
#include <iostream>
#include <math.h>
using namespace std;

FlightMode::FlightMode()
{
    charact = new Characteristics;
}

map<int,int> FlightMode::readCharacteristics(FILE *fp)
{
    map<int,int> ch;
    char buf[256];
    int id, value;

#ifdef DEBUG    
    cout << "----\n";
#endif

    while (fgets(buf, 255, fp) != NULL && buf[0] != '#') {
        sscanf(buf, "%d %d", &id, &value);
        ch[id] = value;

#ifdef DEBUG
        int value0 = ch[id];
        cout << "  CHARACT: " << id << " -> " << value0 << "\n";
#endif

    }
    return ch;
}

int FlightMode::getDistance(Position *pos, Position *gpos)
{
    double dx = gpos->x - pos->x;
    double dy = gpos->y - pos->y;
    int dist = (int)sqrt(dx * dx + dy * dy);

    return dist;
}

int FlightMode::getAngle(Position *pos, int dir, Position *gpos)
{
    double absx = abs(pos->x - gpos->x);
    double absy = abs(pos->y - gpos->y);
    double radian;
    double theta;
    int angle;

    if (absx == 0 || absy == 0) {
        radian = 0;
        theta = 0;
    } else{
        radian = atan(absx / absy);
        theta = radian * (360 / (2 * PI));
    }
#ifdef DEBUG    
    cout << "theta " << theta << "\n";
#endif
    // 1ݸ (Ȳ)
    if (pos->x > gpos->x && pos->y > gpos->y) {
        angle = 180 - 17 - dir - (int)theta;
    }

    // 2ݸ
    else if (pos->x < gpos->x && pos->y > gpos->y) {
       angle = (int)theta - dir + 180 - 17;
    }

    // 3ݸ
    else if (pos->x < gpos->x && pos->y < gpos->y) {
        angle = (int)theta + dir + 17;
    }

    // 4ݸ
    else if (pos->x > gpos->x && pos->y < gpos->y){
        angle = (int)theta - dir - 17;
    }

    // 뿿
    else if (pos->x == gpos->x && pos->y == gpos->y) {
        angle = 0;
    }

    // y塦¦
    else if (pos->x == gpos->x && pos->y > gpos->y) {
        angle = 523 - dir;
    }

    // y塦¦
    else if (pos->x == gpos->x && pos->y < gpos->y) {
        angle = 343 - dir;
    }

    // x塦¦
    else if (pos->x > gpos->x && pos->y == gpos->y) {
        angle = 433 - dir;
    }

    // x塦¦
    else if (pos->x < gpos->x && pos->y == gpos->y) {
        angle = 613 - dir;
    }

    return (angle + 360) % 360;
}

int FlightMode::getAltitude(int z, int gz)
{
    int dz = (gz - z);
    return dz;
}
