/*
 *    LossOfControl.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "LossOfControl.h"
#include <cstdio>
#include <iostream>
using namespace std;

void LossOfControl::setCharacteristics(char *filename)
{
    char name[256];
    strcpy(name, "loss_");
    strcat(name, filename);

    FILE *fp = fopen(name, "r");
    if (fp == NULL) {
        cout << "File not found: " << name << "\n";
        return;
    }

#ifdef DEBUG
    cout << "READING CHARACT: " << name << "\n";
#endif

    charact->distMap = readCharacteristics(fp);
    charact->upperMap = readCharacteristics(fp);
    charact->rightMap = readCharacteristics(fp);
    charact->leftMap = readCharacteristics(fp);

    fclose(fp);
}

bool LossOfControl::hasArrived(Position *pos, Position *gpos)
{
    cout << "LossOfControl: ";
    int dz = abs(gpos->z - pos->z);
    cout << "GOAL = (" << gpos->x << " " << gpos->y << " "<< gpos->z << "): " << dz << "\n";
//    return dz < ALT_ERROR_IN_ARRIVAL;
    return false;
}

MotorPower *LossOfControl::getMotorPower(Position *pos, int dir, Position *gpos)
{
    int altdiff = getAltitude(pos->z, gpos->z);
    return calculate(altdiff);
}

MotorPower *LossOfControl::calculate(int altdiff)
{
    static MotorPower power;

    altdiff /= 10;
    if (charact->upperMap.count(abs(altdiff)) == 1) {
        power.upper = charact->upperMap[abs(altdiff)];
    } else {
        power.upper = MAX_MOTOR_POWER;
    }
    if (altdiff < 0) {
	power.upper *= -1;
    }
    power.upper += HOVER_MOTOR_POWER;

    int lweight = random() % 10 - 5;
    power.left = MIN_MOTOR_POWER * lweight;

    int rweight = random() % 10 - 5;
    power.right = -MIN_MOTOR_POWER * rweight;

    return &power;
}
