/*
 *    NormalFlight.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "NormalFlight.h"
#include <cstdio>
#include <iostream>
#include <string.h>
#include <math.h>
using namespace std;

void NormalFlight::setCharacteristics(char *filename)
{
    char name[256];
    strcpy(name, "normal_");
    strcat(name, filename);

    FILE *fp = fopen(name, "r");
    if (fp == NULL) {
        cout << "File not found: " << name << "\n";
        return;
    }

#ifdef DEBUG
    cout << "READING CHARACT: " << name << "\n";
#endif

    charact->distMap = readCharacteristics(fp);
    charact->upperMap = readCharacteristics(fp);
    charact->rightMap = readCharacteristics(fp);
    charact->leftMap = readCharacteristics(fp);

    fclose(fp);
}

bool NormalFlight::hasArrived(Position *pos, Position *gpos)
{
    cout << "NormalMode: ";
    double dx = gpos->x - pos->x;
    double dy = gpos->y - pos->y;
    int dz = abs(gpos->z - pos->z);
    int xy = (int)sqrt(dx * dx + dy * dy);
    cout << "GOAL = (" << gpos->x << " " << gpos->y << " "<< gpos->z << "): (" << xy << "," << dz << ")\n";

    return xy < DIST_ERROR_IN_ARRIVAL && dz < ALT_ERROR_IN_ARRIVAL;
}

MotorPower *NormalFlight::getMotorPower(Position *pos, int dir, Position *gpos)
{
    int dist = getDistance(pos, gpos);
    int ang = getAngle(pos, dir, gpos);
    int altdiff = getAltitude(pos->z, gpos->z);

    return calculate(dist, ang, altdiff);
}

MotorPower *NormalFlight::calculate(int dist, int ang, int altdiff)
{
    static MotorPower power;

    altdiff /= 10;
    if (charact->upperMap.count(abs(altdiff)) == 1) {
        power.upper = charact->upperMap[abs(altdiff)];
    } else {
        power.upper = MAX_MOTOR_POWER;
    }
    if (altdiff < 0) {
	power.upper *= -1;
    }
    power.upper += HOVER_MOTOR_POWER;

    ang /= 10;
    if (charact->leftMap.count(ang) == 1) {
        power.left = charact->leftMap[ang];
    } else {
        power.left = MIN_MOTOR_POWER;
    }
    if (charact->rightMap.count(ang) == 1) {
        power.right = charact->rightMap[ang];
    } else {
        power.right = MIN_MOTOR_POWER;
    }

    dist /= 10;
    float weight;
    if (charact->distMap.count(dist) == 1) {
        weight = charact->distMap[dist] / 100;  // percentage
    } else {
        weight = 1;
    }
    cout << "ANG " << ang << " ";
    cout << dist << " ";
cout << power.left << " " << power.right << " " << weight << "\n";
    power.left = (int)(power.left * weight);
    power.right = (int)(power.right * weight);

    return &power;    
}
