/*
 *    USReceiver.cpp
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "USReceiver.h"
#include <math.h>

#define BUF_LEN 256

struct ussensor {
    Position pos;
    int time;
};


static Position calcXYZ(struct ussensor *us) {
    Position ret;
    double x1 = us[0].pos.x;
    double y1 = us[0].pos.y;
    double x2 = us[1].pos.x;
    double y2 = us[1].pos.y;
    double x3 = us[2].pos.x;
    double y3 = us[2].pos.y;
    double x4 = us[3].pos.x;
    double y4 = us[3].pos.y;
    double t2 = us[1].time * US_SPEED / 1000;
    double t3 = us[2].time * US_SPEED / 1000;
    double t4 = us[3].time * US_SPEED / 1000;
    double x;
    double y;
    double z;
    double R;
    double A = (x1 - x4) / (double)t4;
    double B = (y1 - y4) / (double)t4;
    double C = (x1 - x3) / (double)t3;
    double D = (y1 - y3) / (double)t3;
    double E = (x1 - x2) / (double)t2;
    double F = (y1 - y2) / (double)t2;
    double M = ((x1 * x1) - (x4 * x4) + (y1 * y1) - (y4 * y4) + (t4 * t4)) /
        (double)(-2 * t4);
    double N = ((x1 * x1) - (x3 * x3) + (y1 * y1) - (y3 * y3) + (t3 * t3)) /
        (double)(-2 * t3);
    double P = ((x1 * x1) - (x2 * x2) + (y1 * y1) - (y2 * y2) + (t2 * t2)) /
        (double)(-2 * t2);
    x = ((M - P) * (D - F) - (N - P) * (B - F)) /
        ((C - E) * (B - F) - (A - E) * (D - F));
    y = ((M - P) * (C - E) - (N - P) * (A - E)) /
        ((D - F) * (A - E) - (B - F) * (C - E));
    R = A * x + B * y + M;

    z = sqrt(fabs(((x - x1) * (x - x1)) + ((y - y1) * (y - y1)) - R * R));

    ret.x = (int)x;
    ret.y = (int)y;
    ret.z = (int)z;

#ifdef VERBOSE
    cout << x << " " << y << " " << z << " " << R << "\n";
#endif

    return ret;
}

static int usstrtoint(char *str) {
    int ret;

    ret = ((digittoint(str[0]) * 16 + digittoint(str[1])) * 16 +
	   digittoint(str[2])) * 16 + digittoint(str[3]);

    return ret;
}

USReceiver::USReceiver()
{
    writer = &positionData;
    data.x = INVALID_VALUE;
    data.y = INVALID_VALUE;
    data.z = INVALID_VALUE;
}

void USReceiver::receive()
{
    /* US󥵤ǡ񤭽Ф */
    char buf[BUF_LEN];
    int size;
    unsigned short int presensor, sensor, tmpsensor;
    int time, sensorno = 0;
    int no = -1;
    struct ussensor ussensor[16];
    presensor = ~0; /*All bit on*/

    while (1) {
	size = USRead(buf, BUF_LEN);
	if (size == -1)
//	    break;
	    continue; // timeout æФʤ
	if (size != 10)
	    continue;
	time = usstrtoint(&(buf[5]));
	if (time == 0) {
	    /* 󥿥ꥻåȤʤǼľ */
	    no = 0;
	    presensor = ~0;
	}
	/* time == 0 ˤʤޤǼľ */
	if (no == -1)
	    continue;
	tmpsensor = usstrtoint(buf);
	sensor =  ~(tmpsensor | ~presensor);
	presensor = tmpsensor;

	for (int i = 0; i < 16; i++) {
	    if ((1 << i & sensor) == sensor) {
		sensorno = 15 - i;
		break;
	    }
	}

	ussensor[no].pos = writer->getUSSensor(sensorno)->pos;
	ussensor[no].time = time / 2;
	no++;
//	if (no == 4) // US 4ĤȤʤΤ
	    break;
    }

#ifdef VERBOSE
    for (int i = 0; i < no; i++) {
	printf("X %d Y %d time %d\n", ussensor[i].pos.x, ussensor[i].pos.y, ussensor[i].time);
    }
#endif
    if (no == 4) {
	data = calcXYZ(ussensor);
    } else if ( no > 0) {
	data.x = ussensor[0].pos.x;
	data.y = ussensor[0].pos.y;
	data.z = ussensor[0].pos.z;
    }
    writer->write(data);
}

#ifdef DEBUG
void USReceiver::receive(int x, int y, int z, unsigned long usec)
{
    data.x = x;
    data.y = y;
    data.z = z;
    writer->write(data);
    cout << "WRITE US_DATA: (" << data.x << "," << data.y << "," << data.z << ")\n";
    usleep(usec);
}
#endif
