/*
 *    fsel-comm.cpp (User-input-data Received Flight Mode)
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "BlimpSystem.h"
#include "AltitudeData.h"
#include "DirectionData.h"
#include "PositionData.h"
#include "RFReceiver.h"
#include "USReceiver.h"
#include "UDPReceiver.h"
#include "BlimpGUI.h"
#include <stdio.h>
#include <iostream>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netdb.h>
#include <stdlib.h>
using namespace std;

BlimpSystem blimpSystem;
AltitudeData altitudeData;
DirectionData directionData;
PositionData positionData;

#define DEFAULT_PORT 15000

void start();
void *runTCPReceiver();
void broken_socket(int sig);

bool thread_running;

int main(int argc, char **argv) {
    blimpSystem.setBlimp(new BlimpGUI(argc, argv));
    // blimpSystem.setBlimp(new FlightController);  // default

    blimpSystem.setUSSensors("ussensor.dat");
    blimpSystem.setCharacteristics("characteristics.dat");
    blimpSystem.setFlightPath("flightpath.dat");

    usleep(1000000);

    cout << "\n\n";
    cout << "**** READY TO FLIGHT (Random Mode) ****\n\n";
    cout << "Please any key to start flighting !\n\n";
    char dummy;
    cin.get(dummy);

    start();
}

void start() {
    pthread_t thread1;
    pthread_t thread4;

    thread_running = 1;
    pthread_create(&thread1, NULL, (void *(*)(void *))runTCPReceiver, (void *)NULL);
    pthread_create(&thread4, NULL, GUIThread, NULL);

    blimpSystem.setSleep(1000000);
    blimpSystem.startAutomaticFlight();

    thread_running = 0;
    pthread_join(thread1, NULL);
    pthread_join(thread4, NULL);
}

void *runTCPReceiver()
{
    int port = DEFAULT_PORT;
    char localhost[64];
    struct hostent *hp;
    struct sockaddr_in sa;
    socklen_t sa_len = sizeof(sa);
    int listen_sock, sock;
    const int nonzero = 1;
    int len;

    int alt, dir, x, y, z;
    char buf[10];
    bool receiving = 0;

    RFReceiver receiver1;
    USReceiver receiver2;
    UDPReceiver receiver3;

    gethostname(localhost, 256);
    strcpy(localhost, "localhost");
    printf("HOST = %s\n", localhost);
    if ((hp = gethostbyname(localhost)) == NULL) {
        perror("server: hostname");
    }
    bcopy((char *)hp->h_addr, (char *)&sa.sin_addr, hp->h_length);
    sa.sin_family = hp->h_addrtype;
    sa.sin_port = port;

    if ((listen_sock = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0) {
        perror("server: socket");
        exit(1);
    }

    setsockopt(listen_sock, SOL_SOCKET, SO_REUSEADDR, &nonzero, sizeof(nonzero));

    if (bind(listen_sock, (struct sockaddr *)&sa, sa_len) < 0) {
        perror("server: bind");
        exit(1);
    }
    listen(listen_sock, 2);
    signal(SIGPIPE, broken_socket);

    while (thread_running) {
        if ((sock = accept(listen_sock, (struct sockaddr *)&sa, &sa_len)) < 0) {
            perror("server: accept");
            exit(1);
        }

        receiving = 1;
        while (receiving) {
            if ((len = read(sock, buf, 6)) <= 0) {
                receiving = 0;
            }
            buf[len] = '\0';
            alt = atoi(buf);

            if ((len = read(sock, buf, 6)) <= 0) {
                receiving = 0;
            }
            buf[len] = '\0';
            x = atoi(buf);

            if ((len = read(sock, buf, 6)) <= 0) {
                receiving = 0;
            }
            buf[len] = '\0';
            y = atoi(buf);

            if ((len = read(sock, buf, 6)) <= 0) {
                receiving = 0;
            }
            buf[len] = '\0';
            z = atoi(buf);

            if ((len = read(sock, buf, 6)) <= 0) {
                receiving = 0;
            }
            buf[len] = '\0';
            dir = atoi(buf);

            receiver1.receive(alt, 0);
            receiver2.receive(x, y, z, 0);
            receiver3.receive(dir, 0);

            fprintf(stdout, "-- %d %d %d %d %d\n", alt, x, y, z, dir);

        }
    }

    close(sock);
    shutdown(sock, 2);
    return NULL;
}

void broken_socket(int sig)
{
    thread_running = 0;
}
