/*
 *    fsel.cpp (Automatic Flight Mode)
 *
 *    Copyright (c) 2005 Software Science and Technology Laboratory,
 *    Ritsumeikan University. All rights reserved.
 */

#include "defs.h"
#include "BlimpSystem.h"
#include "AltitudeData.h"
#include "DirectionData.h"
#include "PositionData.h"
#include "RFReceiver.h"
#include "USReceiver.h"
#include "UDPReceiver.h"
#include "BlimpGUI.h"
#include <iostream>
#include <unistd.h>
using namespace std;

BlimpSystem blimpSystem;
AltitudeData altitudeData;
DirectionData directionData;
PositionData positionData;

void start();
void *runRFReceiver();
void *runUSReceiver();
void *runUDPReceiver();

bool thread_running;

int main(int argc, char **argv) {
    blimpSystem.setBlimp(new BlimpGUI(argc, argv));
    // blimpSystem.setBlimp(new FlightController);  // default

    blimpSystem.setUSSensors("ussensor.dat");
    blimpSystem.setCharacteristics("characteristics.dat");
    blimpSystem.setFlightPath("flightpath.dat");
    blimpSystem.setSleep(500000);
    COMInit();

//    usleep(1000000);

    cout << "\n\n";
    cout << "**** READY TO FLIGHT ****\n\n";
    cout << "Please any key to start flighting !\n\n";
    char dummy;
    cin.get(dummy);

    start();
}

void start() {
    pthread_t thread1;
    pthread_t thread2;
    pthread_t thread3;
    pthread_t thread4;

    thread_running = 1;
    pthread_create(&thread1, NULL, (void *(*)(void *))runRFReceiver, (void *)NULL);
    pthread_create(&thread2, NULL, (void *(*)(void *))runUSReceiver, (void *)NULL);
    pthread_create(&thread3, NULL, (void *(*)(void *))runUDPReceiver, (void *)NULL);
    pthread_create(&thread4, NULL, GUIThread, NULL);

    blimpSystem.startAutomaticFlight();

    thread_running = 0;
    pthread_join(thread1, NULL);
    pthread_join(thread2, NULL);
    pthread_join(thread3, NULL);
    pthread_join(thread4, NULL);
}

void *runRFReceiver() {
    RFReceiver receiver;
    while (thread_running) {
        receiver.receive();
    }
    return NULL;
}

void *runUSReceiver() {
    USReceiver receiver;
    while (thread_running) {
        receiver.receive();
    }
    return NULL;
}

void *runUDPReceiver() {
    UDPReceiver receiver;
    while (thread_running) {
        receiver.receive();
    }
    return NULL;
}
